/* Copyright (c) 2001, Anders Dinsen <anders@dinsen.net>
   All rights reserved.

   Redistribution and use in source and binary forms, with or without
   modification, are permitted provided that the following conditions
   are met:

   * Redistributions of source code must retain the above copyright
     notice, this list of conditions and the following disclaimer.
   * Redistributions in binary form must reproduce the above
     copyright notice, this list of conditions and the following
     disclaimer in the documentation and/or other materials
     provided with the distribution.
   * Neither the name of copyright holders nor the names of the
     contributors may be used to endorse or promote products
     derived from this software without specific prior written
     permission.

   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND
   CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,
   INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
   MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
   DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE
   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
   OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
   PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
   PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
   THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
   TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF
   THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
   SUCH DAMAGE. 
*/

/* a stupid replacement for fgetln(3) for systems that doesn't have it
   (e.g. Linux) */

#include <stdio.h>
#include <stdlib.h>

#include <netdb.h> /* XXX keeping util.h happy */
#include "util.h"  /* needed for our xmalloc() + xrealloc() */

#define OPTIMISTIC 80

char *
fgetln(FILE *fp, size_t *lenp) {
    static char *buf = NULL;
    size_t buflen = 0;
    int ch;

    if ( buf != NULL ) {
        free(buf);
	buf = NULL;
    }

    *lenp = 0;
    do {
        ch = getc(fp);
	if ( ch != EOF ) {
	    if ( buf == NULL ) {
	        buf = xmalloc(OPTIMISTIC);
		buflen = OPTIMISTIC;
	    }
	    if ( *lenp == buflen ) {
		buf = xrealloc(buf, buflen += OPTIMISTIC);
	    }
	    buf[(*lenp)++] = ch;
    	}
    } while ( ch != '\n' && ch != EOF );
    return (buf);
}

